SystemOrganization addCategory: #'GT-Inspector'!

!Number methodsFor: '*GT-Inspector' stamp: 'AndreiChis 4/20/2016 17:55'!
gtInspectorHash

	^ self hash! !

!HelpTopic methodsFor: '*GT-Inspector' stamp: 'AndreiChis 3/6/2015 10:35'!
gtDisplayOn: stream
	^ stream nextPutAll: self title! !

!GlamourPresentationModel class methodsFor: '*GT-Inspector' stamp: 'JurajKubelka 12/27/2016 16:26'!
exampleInspectorOn42
	<gtExample>
	"open an inspector on 42"
	^ self inspectorOn: 42! !

!GlamourPresentationModel class methodsFor: '*GT-Inspector' stamp: 'JurajKubelka 12/27/2016 16:26'!
exampleSimpleMethodsBrowser
	<gtExample>
	"Open a Methods Browser on Object."
	|ui|
	ui := GlamourPresentationModel new.
	ui presentationClass: GTSimpleMethodsBrowser startOn: Object.
	^ ui openWithSpec! !

!GlamourPresentationModel class methodsFor: '*GT-Inspector' stamp: 'JurajKubelka 1/18/2017 16:32'!
inspectorOn: anObject
	"Open an inspector on anObject. 
	Execute the following example:
	self inspectorOn: 42.
	self inspectorOn: self new."
	|ui|
	ui := GlamourPresentationModel new.
	ui presentationClass: GTInspector startOn: anObject.
	^ ui openWithSpec! !

!Morph methodsFor: '*GT-Inspector' stamp: 'AndreiChis 2/26/2017 18:57'!
gtInspectorIcon
	^ self class systemIcon! !

!ProtoObject methodsFor: '*GT-Inspector' stamp: 'TudorGirba 1/4/2016 20:07'!
gtInspectorActionBrowse
	<gtInspectorAction>
	^ GLMGenericAction new
		action: [ Smalltalk tools browser openOnClass: self class ];
		icon: GLMUIThemeExtraIcons glamorousBrowse;
		title: 'Browse'! !

!ProtoObject methodsFor: '*GT-Inspector' stamp: 'AutoDeprecationRefactoring 7/13/2016 09:16'!
gtInspectorActions
	"This is a utility method that allows each object to define a set of actions 
	specific for itself. These actions are used when the object appears in the inspector.

	The actions are collected based on the #gtInspectorAction pragma that are defined in
	unary methods corresponding to the body of the actions"

	| all |
	all := (Pragma
		allNamed: #gtInspectorAction
		from: self class
		to: ProtoObject)
		collect: [ :eachPragma | 
			thisContext
				object: self
				perform: eachPragma methodSelector
				withArguments: #()
				inClass: self class ].
	^ all
		asSortedCollection: [ :a :b | 
			| first second |
			first := a isCategorized
				ifTrue: [ a category , a title ]
				ifFalse: [ a title ].
			second := b isCategorized
				ifTrue: [ b category , b title ]
				ifFalse: [ b title ].
			first < second ]! !

!ProtoObject methodsFor: '*GT-Inspector' stamp: 'AndreiChis 2/26/2017 18:02'!
gtInspectorIcon
	^ nil! !

!ProtoObject methodsFor: '*GT-Inspector' stamp: 'TudorGirba 1/4/2016 21:04'!
gtInspectorMetaIn: composite
	"This presentation offers the methods of the class of the current object"
	<gtInspectorPresentationOrder: 1000>
	<gtInspectorTag: #basic>
	^ composite custom: (GTSimpleMethodsBrowser new startOn: self) ! !

!ProtoObject methodsFor: '*GT-Inspector' stamp: 'AutoDeprecationRefactoring 7/13/2016 09:16'!
gtInspectorPresentationsFromPragmas: aCollection In: composite inContext: aGTInspector
	aCollection
		do: [ :eachPragma | 
			eachPragma methodSelector numArgs = 0
				ifTrue: [ | presentationSource |
					presentationSource := self perform: eachPragma selector.
					presentationSource glmPresentation
						cull: composite
						cull: aGTInspector
						cull: self ].
			eachPragma methodSelector numArgs = 1
				ifTrue: [ thisContext
						object: self
						perform: eachPragma methodSelector
						withArguments: {composite}
						inClass: self class ].
			eachPragma methodSelector numArgs = 2
				ifTrue: [ thisContext
						object: self
						perform: eachPragma methodSelector
						withArguments:
							{composite.
							aGTInspector}
						inClass: self class ] ]! !

!ProtoObject methodsFor: '*GT-Inspector' stamp: 'AliakseiSyrel 2/9/2017 00:19'!
gtInspectorPresentationsIn: composite inContext: aGTInspector
	"This is a utility method that collects all presentations for the current object.
	By default, it simply looks for the #gtInspectorPresentationOrder: pragma.
	The inspector can decice whether or not a presentation should be dispayed.
	Subclasses might want to override it for more special behavior."
	| pragmas |
	pragmas := Pragma
		allNamed: aGTInspector class extensionsPragma
		from: self class
		to: ProtoObject
		sortedUsing: [ :x :y | (x argumentAt: 1) < (y argumentAt: 1) ].
	pragmas := pragmas select: [ :aPragma | aGTInspector shouldDisplayPresentationCreatedBy: aPragma method ].
	self gtInspectorPresentationsFromPragmas: pragmas In: composite inContext: aGTInspector! !

!ProtoObject methodsFor: '*GT-Inspector' stamp: 'TudorGirba 1/4/2016 20:41'!
gtInspectorRawIn: composite
	"This is the most basic presentation showing the state of the object"
	<gtInspectorPresentationOrder: 10>
	<gtInspectorTag: #basic>
	^ composite custom: GTProtoObjectVariablesBrowser new! !

GLMCompositePresentation subclass: #GTInspector
	instanceVariableNames: 'presentationFilter'
	classVariableNames: 'EnabledStepRefreshStatus GTInspectorEnabledStatus GTNumberOfInspectorPanes IndexableDisplayLimit PreferredExtent RememberPreferredExtent StepRefreshRate'
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspector commentStamp: 'TudorGirba 8/21/2011 16:57' prior: 0!
This is meant to be used for inspecting objects. It offers multiple views and it uses a finder as a navigation. One particular feature is that you can use the evaluator tab to enter code, and evaluating it results in opening another pane to the right.

The object finder asks dynamically the object for the actual presentations that are displayed in each pane.

Example:
self openOn: Smalltalk.

Register it as a replacement for explorer:
self registerToolsOn: Smalltalk tools.!

!GTInspector class methodsFor: 'utilities api' stamp: 'MarcusDenker 2/17/2017 10:08'!
closeAllInspectors
	<script>
	"self closeAllInspectors"
	(SystemWindow allSubInstances select: [:w | 
		 w model isKindOf: self])
			do: [:w | w delete ].
! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 3/3/2017 00:57'!
disableStepRefresh
	self setEnabledStepRefreshStatus: false! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 3/3/2017 00:57'!
enableStepRefresh
	self setEnabledStepRefreshStatus: true! !

!GTInspector class methodsFor: 'openning' stamp: 'TudorGirba 11/1/2013 22:16'!
explore: anObject 
	self openOn: anObject.
	^ anObject! !

!GTInspector class methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 16:09'!
extensionsPragma
	^ #gtInspectorPresentationOrder:! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:38'!
extentSettingsOn: aBuilder 
	<systemsettings>
	
	(aBuilder setting: #GTInspectorPreferredWidth)
		target: self;
		parent: #gt;
		getSelector: #preferredWidth;
		setSelector: #preferredWidth:; 
		label: 'Inspector width';
		description: 'The preferred width of new Inspector windows'.
	(aBuilder setting: #GTInspectorPreferredHeight)
		target: self;
		parent: #gt;
		getSelector: #preferredHeight;
		setSelector: #preferredHeight:; 
		label: 'Inspector height';
		description: 'The preferred height of new Inspector windows'.
	(aBuilder setting: #GTInspectorRememberPreferredExtent)
		target: self;
		parent: #gt;
		getSelector: #rememberPreferredExtent;
		setSelector: #rememberPreferredExtent:; 
		label: 'Remember extent of Inspector';
		description: 'Enable or disable remembering of the extent of Inspector windows on resize. When the preference is true, resizing a Inspector window will affect the preferred width and height of future Inspector windows'
! !

!GTInspector class methodsFor: 'inspecting' stamp: 'TudorGirba 3/8/2014 07:51'!
gtInspectorExtensionsIn: composite
	<gtInspectorPresentationOrder: 40>
	composite list
		title: 'Extensions';
		display: [ self inspectorExtendingMethods ];
		showOnly: 50;
		format: [ :each | each gtDisplayString ];
		filterOn: [ :text :each | each gtDisplayString includesSubstring: text caseSensitive: false ]! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 2/26/2017 19:38'!
indexableDisplayLimit
	^ IndexableDisplayLimit ifNil: [ 5000 ]! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 2/26/2017 19:37'!
indexableDisplayLimit: anInteger
	IndexableDisplayLimit := anInteger! !

!GTInspector class methodsFor: 'class initialization' stamp: 'TudorGirba 10/7/2012 22:50'!
initialize 
	self numberOfInspectorPanes: 2! !

GTInspector initialize!

!GTInspector class methodsFor: 'openning' stamp: 'TudorGirba 11/1/2013 22:16'!
inspect: anObject 
	self openOn: anObject.
	^ anObject! !

!GTInspector class methodsFor: 'openning' stamp: 'AndreiChis 4/16/2014 18:15'!
inspect: anObject label: aString
	self new 
		title: aString;
		openOn: anObject.
	^ anObject
! !

!GTInspector class methodsFor: 'openning' stamp: 'AndreiChis 10/28/2014 13:53'!
inspector: anObject

	^ self openOn: anObject! !

!GTInspector class methodsFor: 'inspecting' stamp: 'TudorGirba 6/25/2016 09:01'!
inspectorExtendingMethods
	^ (SystemNavigation default allBehaviors flatCollect: [ :each | 
			(Pragma allNamed: self extensionsPragma in: each) 
				collect: [ :p | p method asRingDefinition ] ])
		sorted! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 8/29/2014 00:48'!
isGTInspectorEnabled

	^ GTInspectorEnabledStatus ifNil: [ GTInspectorEnabledStatus := Smalltalk tools inspector = self ]! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 3/3/2017 00:57'!
isStepRefreshEnabled

	^ EnabledStepRefreshStatus ifNil: [ false ]! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 10/7/2012 22:46'!
numberOfInspectorPanes
	^ GTNumberOfInspectorPanes ! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 10/7/2012 22:46'!
numberOfInspectorPanes: anInteger
	GTNumberOfInspectorPanes := anInteger ! !

!GTInspector class methodsFor: 'openning' stamp: 'TudorGirba 11/25/2012 22:53'!
openOn: anObject 
	^ self new openOn: anObject! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:36'!
preferredExtent
	^ PreferredExtent ifNil: [ PreferredExtent := 600@400 ]! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:37'!
preferredExtent: aPoint
	PreferredExtent := aPoint! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:37'!
preferredHeight
	^ self preferredExtent y! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:37'!
preferredHeight: aHeight
	self preferredExtent setX: self preferredExtent x setY: aHeight! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:37'!
preferredWidth
	^ self preferredExtent x! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:37'!
preferredWidth: aWidth
	self preferredExtent setX: aWidth setY: self preferredExtent y! !

!GTInspector class methodsFor: 'tools registry' stamp: 'AndreiChis 8/28/2014 21:21'!
register

	self registerToolsOn: Smalltalk tools
! !

!GTInspector class methodsFor: 'tools registry' stamp: 'TudorGirba 11/1/2013 22:17'!
registerToolsOn: registry
	"self registerToolsOn: Smalltalk tools " 
	registry register: self as: #inspector
! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:37'!
rememberPreferredExtent
	^ RememberPreferredExtent ifNil: [ RememberPreferredExtent := true ]! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:37'!
rememberPreferredExtent: aBoolean
	^ RememberPreferredExtent := aBoolean! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 3/3/2017 00:56'!
setEnabledStepRefreshStatus: aBoolean
	EnabledStepRefreshStatus := aBoolean! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 8/28/2014 21:27'!
setGTInspectorEnabledStatus: aBoolean
	| oldStatus |
	
	oldStatus := self isGTInspectorEnabled.
	GTInspectorEnabledStatus := aBoolean.
	oldStatus ~= GTInspectorEnabledStatus ifTrue: [ 
		GTInspectorEnabledStatus 
			ifTrue: [ self register ]
			ifFalse: [ EyeInspector registerToolsOn: Smalltalk tools ] ]
	
	! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 3/26/2015 00:40'!
setPreferredExtentIfWanted: anExtent
	self rememberPreferredExtent ifTrue: [ 
		self preferredExtent: anExtent ]! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 3/3/2017 01:01'!
settingsOn: aBuilder 
	<systemsettings>
	
	(aBuilder group: #gt)
		parent: #tools;
		label: 'Glamorous Toolkit';
		with: [
			(aBuilder setting: #numberOfInspectorPanes)
				target: self; 
				label: 'Number of inspector panes';
		 		description: 'Specifies the maximum number of panes that are visible at every moment in a Glamorous Inspector.'.
			(aBuilder setting: #GTInspectorStatus)
				target: self;
				getSelector: #isGTInspectorEnabled;
				setSelector: #setGTInspectorEnabledStatus:; 
				label: 'GTInspector';
		 		description: 'Enable or disable the GTInspector.'.
			(aBuilder setting: #EnabledStepRefreshStatus)
				target: self;
				getSelector: #isStepRefreshEnabled;
				setSelector: #setEnabledStepRefreshStatus:; 
				label: 'GTInspector Step Refresh';
		 		description: 'Enable or disable the automatic refresh in GTInspector that uses a busy update loop (based on the stepping mechanism from Morphic).']! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 2/26/2017 22:41'!
stepRefreshRate
	^ StepRefreshRate ifNil: [ 0 ]! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 2/26/2017 22:41'!
stepRefreshRate: anInteger
	StepRefreshRate := anInteger! !

!GTInspector methodsFor: 'callbacks' stamp: 'AliakseiSyrel 2/9/2017 00:20'!
actOnPaneAdded: anAnnouncement
	
	"Update toolbar of a pane before the last (added) pane if there is one"
	(self panes atLast: 2 ifAbsent: [ ^ self ])
		updateToolbar! !

!GTInspector methodsFor: 'callbacks' stamp: 'AliakseiSyrel 2/9/2017 00:28'!
actOnPaneRemoved: anAnnouncement

	"Update toolbar of a pane before the removed pane (now it is last one)"
	self panes last updateToolbar! !

!GTInspector methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/9/2017 00:22'!
browser
	^ self first! !

!GTInspector methodsFor: 'actions' stamp: 'AliakseiSyrel 2/9/2017 00:05'!
closeActionFor: aPresentation

	^ GLMGenericAction new
			action: [ self removeLastPane ];
			icon: GLMUIThemeExtraIcons glamorousClose;
			condition: [ :currentPresentation |
				self panes size > 1 and: [ self panes last presentations = currentPresentation ] ];
			title: 'Close this inspector column'! !

!GTInspector methodsFor: 'building' stamp: 'AndreiChis 3/3/2017 01:03'!
compose
	self title: [:anObject | 'Inspector on ', (self printObjectAsAnItem: anObject)].
	self act: [:b | b update] icon: GLMUIThemeExtraIcons glamorousRefresh entitled: 'Update'.
	self act: [:b | 
		HelpBrowser open selectTopicSatisfying: [:each |
			each owner notNil and:  [
			(each owner key = GTInspectorHelp key) and: [
				each title = 'Overview' ] ] ] ] 
		icon: GLMUIThemeExtraIcons glamorousHelp 
		entitled: 'Help'.
	self act: [:b | b presentationFilter changeFilterRequestFor: b ] entitled: 'Filter presentations'.
	self pager with: [ :browser | 
		browser fixedSizePanes: self class numberOfInspectorPanes.
		browser show: [ :a :each |
			a title: [ self printObjectAsAnItem: each ].
			a dynamicActions: [ :aPresentation | 
				each gtInspectorActions asOrderedCollection ].
			a titleAction: [ :aPresentation | self closeActionFor: aPresentation ].
			a dynamic 
				display: [ :x | 
					(GLMCompositePresentation new with: [:d | 
						each 
							gtInspectorPresentationsIn: d 
							inContext: self ])
							startOn: x ] ] ].
			
	self browser when: GLMPaneAdded send: #actOnPaneAdded: to: self.
	self browser when: GLMPaneRemoved send: #actOnPaneRemoved: to: self.
	
	self class isStepRefreshEnabled ifTrue: [ 
		self wantsAutomaticRefresh: true.
		self wantsSteps: true.
		self stepTime: self class stepRefreshRate ].
	! !

!GTInspector methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/9/2017 00:21'!
entities
	"returns the list of objects from the #entity port of each pane of the finder"
	^ self panes collect: [ :each | (each port: #entity) value ]! !

!GTInspector methodsFor: 'context' stamp: 'AliakseiSyrel 2/9/2017 00:18'!
filterByMethods: aCollection
	| filter |
	
	filter := GTInspectorMethodListFilter new.
	aCollection do: [ :aMethod |
		filter addAndSelectSignatureFor: aMethod ].
	self presentationFilter: filter! !

!GTInspector methodsFor: 'accessing' stamp: 'AndreiChis 1/30/2015 11:35'!
firstPresentation

	^ self panes first presentations first! !

!GTInspector methodsFor: 'initialization' stamp: 'AndreiChis 1/27/2015 23:30'!
initialize
	super initialize.
	
	self presentationFilter: (GTInspectorTagFilter new
			defaultTag: #custom;
			addAndSelectTag: #basic;
			addAndSelectTag: #custom;
			addAndSelectTag: #examples)! !

!GTInspector methodsFor: 'actions' stamp: 'TudorGirba 8/21/2011 08:56'!
openExplorerFor: anObject
	"this method is needed for compatibility with ObjectExplorer"
	self openOn: anObject! !

!GTInspector methodsFor: 'scripting opening' stamp: 'TudorGirba 4/3/2015 12:53'!
openOn: aPage
	| window |
	window := super openOn: aPage.
	window extent: self class preferredExtent.
	window when: WindowResizing do: [ 
		(window isMinimized or: [ window isMaximized ]) ifFalse: [ 
			"we only wnat to react to real resizing,
			not to minization or maximization"
			self class setPreferredExtentIfWanted: window extent ] ].
	^ window! !

!GTInspector methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/9/2017 00:22'!
panes
	^ self browser panes! !

!GTInspector methodsFor: 'context' stamp: 'AliakseiSyrel 2/9/2017 00:10'!
presentationFilter
	"Return a filter for limiting the number of visible presentations.
	Filter can not be nil"
	<return: #GTInspectorPresentationFilter>

	^ presentationFilter! !

!GTInspector methodsFor: 'context' stamp: 'AliakseiSyrel 2/9/2017 00:17'!
presentationFilter: aPresentationFilter
	"Set a filter for limiting the number of visible presentations.
	Presentation filter must not be nil. If you don't need any filter use GTInspectorNoneFilter instead"
	
	self
		assert: [ aPresentationFilter isNotNil ]
		description: [ 'Presentation filter must not be nil' ].
	
	presentationFilter := aPresentationFilter! !

!GTInspector methodsFor: 'private' stamp: 'AndreiChis 2/12/2017 14:52'!
printObjectAsAnItem: anObject
	| header printString |
	header := (anObject class name first isVowel
		ifTrue: [ 'an ' ]
		ifFalse: [ 'a ' ]) , anObject class name.
	printString := (GTObjectPrinter asTruncatedTextFrom: anObject) asString.
	^ (printString beginsWith: header)
		ifTrue: [ printString ]
		ifFalse: [ 
			String streamContents: [ :out | 
				out << header; << ' ('; << printString; << ')' ] ]! !

!GTInspector methodsFor: 'updating' stamp: 'AndreiChis 3/3/2017 00:42'!
refreshPanes
	self panes do: [ :aPane | 
		aPane updateToolbar.
		aPane presentations presentations first withAllPresentations do: [ :aPresentation |
			aPresentation wantsAutomaticRefresh ifTrue: [ 
				aPresentation requestRefresh ] ]  ]! !

!GTInspector methodsFor: 'actions' stamp: 'AliakseiSyrel 2/9/2017 00:26'!
removeLastPane
	| lastPane |
	
	lastPane := self panes last.
	self browser removeLastPane.
	self browser announce: (GLMPaneRemoved pane: lastPane fromBrowser: self browser)! !

!GTInspector methodsFor: 'updating' stamp: 'AndreiChis 3/3/2017 00:42'!
requestRefresh
	self updateToolbar.
	self refreshPanes! !

!GTInspector methodsFor: 'context' stamp: 'AliakseiSyrel 2/9/2017 00:20'!
shouldDisplayPresentationCreatedBy: aMethod
	"Verify whether or not the presentation created by the given method should be displayed.
	If no filter is given display all presentations."

	^ self presentationFilter shouldDisplayPresentationCreatedBy: aMethod! !

!GTInspector methodsFor: 'scripting' stamp: 'AliakseiSyrel 2/9/2017 00:34'!
showFirst: aBlock

	self browser showFirst: aBlock! !

!GTInspector methodsFor: 'stepping' stamp: 'AndreiChis 3/3/2017 00:47'!
step
	self window ifNil: [ ^ self ].
	self window isActive ifFalse: [ ^ self ].
	
	self wantsAutomaticRefresh ifTrue: [
		self requestRefresh ]! !

GLMCompositePresentation subclass: #GTRawObjectVariablesBrowser
	instanceVariableNames: 'cachedHashes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTRawObjectVariablesBrowser commentStamp: 'TudorGirba 1/4/2016 20:45' prior: 0!
This is a helper browser for the Raw view in the GTInspector. It provides an overview of the slots and it offers an evaluator.!

GTRawObjectVariablesBrowser subclass: #GTObjectVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTObjectVariablesBrowser commentStamp: 'TudorGirba 8/22/2011 12:52' prior: 0!
This is a helper browser for the main view in the GTInspector.

Example:
self openOn: Smalltalk!

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/7/2017 14:00'!
basicVariableNodesFor: anObject in: aCollection

	^ anObject gtInspectorVariableNodesIn: aCollection ! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/20/2017 11:00'!
printArray: anArray
	"The #printString method of an array needs to scan through the entire array to
	determine how to print the array. Only scan through a limited number of elements
	for large arrays."
	| limitedString limit |
	limit := 50000.
	limitedString := String streamContents: [ :aStream | 
		anArray gtInspectorFastPrintOn: aStream.
	] limitedTo: limit.
	limitedString size < limit ifTrue: [^ limitedString].
	^ limitedString , '...etc...'
	
	! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/12/2017 16:06'!
printStringOf: anObject 
	^ ["Treat arrays differently as their #printString method can be slow.
		In the future if multiple objects need to be handled differenty,
		dispatch through the object."
		((anObject class == Array  
			ifTrue: [ self printArray: anObject ]
			ifFalse: [ anObject printString ]) contractTo: 200) asComment] 
		on: Error 
		do: ['"Error printing. Try self printString to debug"']! !

GTRawObjectVariablesBrowser subclass: #GTProtoObjectVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTProtoObjectVariablesBrowser commentStamp: 'TudorGirba 1/4/2016 20:48' prior: 0!
This is a specialization that works with ProtoObject and all its subclasses that except for Object that has its own specialization.

Example:
self openOn: ProtoObject new!

!GTProtoObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/7/2017 14:25'!
basicVariableNodesFor: anObject in: aCollection

	aCollection addAll: ((anObject class allInstVarNames 
			collectWithIndex: [ :ivName :index | 
				GTInspectorProtoObjectNode hostObject: anObject index: index slotName: ivName]) 
					sort: [:aNode :anotherNode | aNode label < anotherNode label ])
			! !

!GTProtoObjectVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 1/4/2016 20:54'!
printStringOf: anObject
	| className |
	className := anObject class name.
	^ (String streamContents: [ :s |
		s
			nextPutAll: (className first isVowel ifTrue: ['an '] ifFalse: ['a ']);
			nextPutAll: className ]) asComment! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/7/2017 14:11'!
basicVariableNodesFor: anObject

	^ #()! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/7/2017 14:00'!
basicVariableNodesFor: anObject in: aCollection

	self subclassResponsibility ! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/26/2017 18:46'!
childrenVariableNodesForNode: aNode
	| variableNodes |

	"Nil should have now a single child: `self`. For now this is needed for
	a smooth navigation in the tree widget."
	"anObject ifNil: [ ^ #() ]."

	variableNodes := OrderedCollection new: aNode value class allSlots size.
	variableNodes add: (GTInspectorWrapperNode hostObject: aNode label: 'self').
	self basicVariableNodesFor: aNode value in: variableNodes.
	^ variableNodes! !

!GTRawObjectVariablesBrowser methodsFor: 'building' stamp: 'AndreiChis 2/27/2017 10:32'!
compose
	self title: 'Raw'.
	self tabulator with: [ :browser |
		browser row: #variables; row: #evaluator.
		browser transmit 
			to: #variables; 
			andShow: [:a | self variablesIn: a ].
		browser transmit 
			to: #evaluator; 
			andShow: [:a | self evaluatorIn: a ].
		browser transmit 
			from: #variables; 
			toOutsidePort: #selection; 
			transformed: [ :aNode | aNode ifNotNil: [aNode value]].
		browser transmit 
			from: #evaluator; 
			toOutsidePort: #selection ].
	self wantsAutomaticRefresh: true! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 4/28/2016 11:09'!
evaluatorIn: composite
	^ composite pharoScript
		doItReceiver: [ :object | object ];
		smalltalkClass: [ :object | object class ];
		display: [ :object | 
			String streamContents: [ :stream | 
				stream 
					nextPutAll: (self printStringOf: object);
					cr;
					nextPutAll: (GTSnippets snippetAt: object class)] ];
		onChangeOfPort: #text act: [ :textPresentation :object |
				GTSnippets 
					snippetAt: object class 
					put: (self stringWithoutInitialCommentFrom: textPresentation text asString).
				 textPresentation clearUserEdits ];
		installDefaultSelectionActions! !

!GTRawObjectVariablesBrowser methodsFor: 'private - updating' stamp: 'AndreiChis 2/7/2017 13:49'!
generateCacheFor: anObject
	| valuesCache |
	
	valuesCache := OrderedDictionary new.
	((self basicVariableNodesFor: anObject) do: [ :assoc | 
		valuesCache 
			at: assoc key 
			put: (self generateHashFor: assoc value) ]).
	
	^ valuesCache! !

!GTRawObjectVariablesBrowser methodsFor: 'private - updating' stamp: 'AndreiChis 2/7/2017 13:49'!
generateHashFor: anObject 

	^ (self basicVariableNodesFor: anObject)
		inject: anObject gtInspectorHash
		into: [ :hashValue :assoc | hashValue bitXor: assoc value gtInspectorHash ].
	
	! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/26/2017 18:01'!
iconFor: anObject
		
	^ [ (anObject gtInspectorIcon)
		ifNil: [ anObject class systemIcon ] ]
			on: Error do: [ :error | self iconFor: anObject error: error ]! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'StephaneDucasse 10/12/2016 22:35'!
iconFor: anEyeElement error: error
	^ self iconNamed: #smallWarningIcon! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 1/4/2016 20:44'!
printStringOf: anObject
	self subclassResponsibility! !

!GTRawObjectVariablesBrowser methodsFor: 'private - updating' stamp: 'AndreiChis 4/20/2016 17:32'!
rebuildCacheFor: anObject

	cachedHashes := self generateCacheFor: anObject! !

!GTRawObjectVariablesBrowser methodsFor: 'stepping' stamp: 'AndreiChis 3/3/2017 00:42'!
requestRefresh
	(self presentations first paneNamed: #variables) presentations first requestRefresh! !

!GTRawObjectVariablesBrowser methodsFor: 'private - updating' stamp: 'AndreiChis 4/20/2016 17:30'!
shouldRefresh
	|newCache result|

	newCache := self generateCacheFor: self entity.
	result := cachedHashes ~= newCache.
	cachedHashes := newCache.
	^ result! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'AliakseiSyrel 3/24/2016 01:00'!
stringWithoutInitialCommentFrom: aString
	^ (aString trimLeft isEmpty not and: [ aString trimLeft first = $" ])
		ifFalse: [ aString trimRight ]
		ifTrue: [|startIndex doubleQuoteCounter|
			startIndex := aString findAnySubstring: '"' startingAt: (aString indexOf: $") + 1.
			doubleQuoteCounter := 1.
			"we need to iterate over the whole string queuing quotes to support nested double quotes"
			[ startIndex <= aString size
				and: [ doubleQuoteCounter % 2 = 1
					or: [ (aString at: startIndex) = $"
						or: [ (aString at: startIndex) isSeparator ] ] ] ]
			whileTrue: [ 
				(aString at: startIndex) = $" ifTrue: [ 
					doubleQuoteCounter := doubleQuoteCounter + 1 ].
				startIndex := startIndex + 1 ].
	
			(startIndex >= (aString size + 1)) ifTrue: [ "No content if a comment cannot be identified. " ^ '' ].
			(aString
				copyFrom: startIndex
				to: aString size) trimBoth ]! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'AliakseiSyrel 2/8/2017 23:51'!
updateInstanceVariable: aSymbol from: anObject basedOn: aString
	| aResult |

	self
		assert: [ anObject class hasInstVarNamed: aSymbol ]
		description: [ 'Instance variable ', aSymbol asString, ' must exist in ', anObject asString ].

	aResult := anObject class compiler
			source: aString;
			context: nil;
			receiver: anObject;
			failBlock: [ ^ self ];
			evaluate.
						
	anObject writeSlotNamed: aSymbol value: aResult! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/12/2017 15:29'!
variableNodesFor: anObject
	| variableNodes |

	"Nil should have now a single child: `self`. For now this is needed for
	a smooth navigation in the tree widget."
	"anObject ifNil: [ ^ #() ]."

	variableNodes := OrderedCollection new: anObject class allSlots size.
	variableNodes add: (GTInspectorSelfNode hostObject: anObject).
	self basicVariableNodesFor: anObject in: variableNodes.
	^ variableNodes! !

!GTRawObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 3/4/2017 11:51'!
variablesIn: composite
	^ composite fastTreeTable
		hasChildren: [ :aNode | aNode key ~= 'self' ];
		children: [ :aNode | aNode childrenNodesFor: self ];
		icon: [ :aNode | self iconFor: aNode value ];
		display: [ :anObject | 
			self variableNodesFor: anObject ];
		column: 'Variable' translated
			evaluated: [ :aNode | (GTObjectPrinter asNonTruncatedTextFrom: aNode label) ];
		column: 'Value' translated
			evaluated: [ :aNode | | rawValue displayValue |
				[ rawValue := aNode rawValue. ] 
					on: Error 
					do: [ displayValue := Text string: aNode errorMessage attribute: TextColor red ].
				displayValue ifNil: [ GTObjectPrinter asTruncatedTextFrom: rawValue ] ];
		morphicSelectionAct: [:list | list selection value browse ] 
			icon: GLMUIThemeExtraIcons glamorousBrowse 
			on: $b 
			entitled: 'Browse' translated;
		morphicSelectionAct: [:list | list selection value inspect ] 
			icon: GLMUIThemeExtraIcons glamorousInspect 
			on: $i 
			entitled: 'Inspect' translated;
		dynamicActionsOnSelection: [ :presentation |
			(presentation rawSelection isKindOf: GTInspectorVariableNode) 
				ifTrue: [presentation rawSelection inspectorActionsForNode ]
				ifFalse: [ #() ] ];
		selectionAct: [:list | 
			((list selection value pointersToExcept: { list selection value })
									reject: [ :each | each pointsOnlyWeaklyTo: list selection value ]) inspect ] 
			on: $t
			entitled: 'Open pointers to' translated
	! !

GLMCompositePresentation subclass: #GTSimpleMethodsBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTSimpleMethodsBrowser commentStamp: 'TudorGirba 1/18/2014 14:29' prior: 0!
This browser is used to browse the class structure of every object in the inspector.

Example:
self openOn: World!

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 11/24/2013 08:37'!
accumulatedSourceOf: m for: object in: composite
	composite accumulator with: [ :acc | 
		acc show: [ :a |
			a title: [ :method | method selector ].
			self sourceOf: m for: object in: a ] ]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'AndreiChis 2/5/2017 18:24'!
classesIn: a
	^ a fastList
		display: [ :object | 
			| all |
			all := OrderedCollection new.
			object class withAllSuperclassesDo: [:each | all add: each ].
			all ];
"		initialize: [ :list | list selection: list entity class ];"
		format: [ :class | class name ];
		morphicSelectionAct: [ :list | Smalltalk tools browser openOnClass: list selection ]
			icon: GLMUIThemeExtraIcons glamorousBrowse
			on: $b
			entitled: 'Browse'! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 3/9/2014 08:59'!
compose
	self title: 'Meta'.
	self tabulator with: [ :browser |
		browser
			row: [ :r | r column: #classes; column: #methods span: 3 ];
			row: #source.
		browser transmit
			to: #classes;
			andShow: [ :a | self classesIn: a ].
		browser transmit
			to: #classes port: #selection;
			transformed: [:each | each class ].
		browser transmit
			fromOutsidePort: #entity;
			from: #classes;
			to: #methods;
			andShow: [ :a | self methodsIn: a ].
		browser transmit
			from: #methods;
			fromOutsidePort: #entity;
			to: #source;
			when: [ :method | method notNil ];
			andShow: [ :a :method :object | self sourceOf: method for: object in: a ].
		browser transmit
			toOutsidePort: #selection;
			from: #methods port: #evaluatedObject.
		browser transmit
			toOutsidePort: #selection;
			from: #source port: #evaluatedObject.
	]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 10/3/2014 15:14'!
evaluationAction
	^ GLMGenericAction new
		action: [ :listPresentation :obj | 
					| result |
					result := obj perform: listPresentation selection selector.
					(listPresentation pane port: #evaluatedObject) value: result ];
		shortcut: $g;
		title: 'Do it and go';
		yourself! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 7/15/2012 18:58'!
methodActionsFor: aMethod
	^ aMethod selector isUnary
		ifTrue: [ Array with: self evaluationAction ]
		ifFalse: [ #() ]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 2/24/2016 19:48'!
methodsIn: a
	^ a list
		display: [ :object :class | class methods collect: #asRingDefinition];
		filterOn: [ :text :each | (text , '*') match: each selector ];
		helpMessage:  'Wildcard filtering (e.g., "*pattern*")';
		tags: [ :method | method category ];
		sorted: [ :method1 :method2 | method1 selector < method2 selector ];
		format: [ :method | method selector ];
		morphicSelectionAct: [ :list :object :class | 
			Smalltalk tools browser 
				openOnClass: class 
				selector: list selection selector ]
			icon: GLMUIThemeExtraIcons glamorousBrowse
			on: $b
			entitled: 'Browse';
		dynamicActionsOnSelection: [ :list | self methodActionsFor: list selection ]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'AndreiChis 10/13/2015 21:30'!
sourceOf: method for: object in: composite
	^ composite pharoMethod 
		display: [ :m | m sourceCode ];
		smalltalkClass: [ :m | m methodClass ];
		doItReceiver: [ object ];
		selectionPopulate: #evaluatedObject
			on: $g
			entitled: 'Do it and go'
			with: [ :text | Smalltalk compiler evaluate: text selectedText for: object logged: false ];
		act: [ :text :m | self updateOuterPaneFrom: text for: method ]
			icon: GLMUIThemeExtraIcons glamorousAccept
			on: $s
			entitled: 'Accept';
		act: [ :text :m | 
			Smalltalk tools browser 
				openOnClass: m methodClass 
				selector: m selector ]
			icon: GLMUIThemeExtraIcons glamorousBrowse
			entitled: 'Browse';
		act: [ :text :m | 
			(text pane port: #evaluatedObject) value: m ]
			icon: GLMUIThemeExtraIcons glamorousInspect
			entitled: 'Inspect'! !

!GTSimpleMethodsBrowser methodsFor: 'private' stamp: 'TudorGirba 12/20/2016 07:23'!
updateOuterPaneFrom: text for: method
	"this is a utility method to tell the pane surrounding the browser to update.
	The reason we do that is to support the use case of updating the GTInspector from
	the editor by modifying code directly in the Inspector."
	
	| outerPane newMethod |
	newMethod := method methodClass compile: text text notifying: nil.
	outerPane := text pane browser pane.
	outerPane browser panes do: #update.
	self
		flag:
			'This is a huge hack because of so many reasons:
				- the presentation accesses the state of the outer browser directly
				- the presentation depends on the implementation of the browser 
				It should be solved through transmissions.
				
				But, it works for now and the browser is reasonably updated'.
	^ outerPane
		withAllPanesAndPresentationsDo: [ :child | 
			(child isKindOf: self class)
				ifTrue: [ 
					((child first paneNamed: #classes) port: #selection) value: method methodClass.
					((child first paneNamed: #methods) port: #selection)
						value:
							((child first paneNamed: #methods) presentations first displayedValue ifNotNil: [ :v | v detect: [ :each | each selector = newMethod ] ]) ] ]! !

!Array methodsFor: '*GT-Inspector' stamp: 'AndreiChis 2/20/2017 11:00'!
gtInspectorFastPrintOn: aStream
	"Only scan through a limited number of elements to determine how to print the Array."
	((1 to: (self size min: self gtCollectionSizeThreshold)) allSatisfy: [ :anIndex | 
			(self at: anIndex) shouldBePrintedAsLiteral ]) ifTrue: [
		self printAsLiteralFormOn: aStream.  ^ self].
	((1 to: (self size min: self gtCollectionSizeThreshold)) allSatisfy: [ :anIndex | 
			(self at: anIndex) isSelfEvaluating ])  ifTrue: [
		self printAsSelfEvaluatingFormOn: aStream. ^ self].

	super printOn: aStream! !

!Collection methodsFor: '*GT-Inspector' stamp: 'AndreiChis 4/20/2016 17:59'!
gtInspectorHash

	^ super gtInspectorHash bitXor: self size hash! !

!Collection methodsFor: '*GT-Inspector' stamp: 'TudorGirba 6/28/2014 22:19'!
gtInspectorInterestingObject
	^ self size = 1
		ifTrue: [ self anyOne ]
		ifFalse: [ self ]! !

CustomHelp subclass: #GTInspectorHelp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorHelp class methodsFor: 'accessing' stamp: 'TudorGirba 1/20/2016 16:45'!
bookName 
	^'GT Inspector'! !

!GTInspectorHelp class methodsFor: 'documentation' stamp: 'AlexandreBergel 12/16/2015 13:43'!
contact
	^ HelpTopic 
		title: 'Contact'
		contents: 
'This Help has been written by Alexandre Bergel <alexandre.bergel AT me.com>. Feel free to get in touch. December 16, 2015.'! !

!GTInspectorHelp class methodsFor: 'documentation' stamp: 'TudorGirba 1/20/2016 23:40'!
extensions
		
	^ HelpTopic 
		title: 'Extensions'
		contents: 'Inspector allows the developer to define custom presentations in different classes. These custom presentations take effect when the Inspector reaches an instance of the extended class. You can find below the list of extensions from this image.
		
You can also browse this list by inspecting:

		GTInspector inspectorExtendingMethods

', self extensionsString! !

!GTInspectorHelp class methodsFor: 'documentation' stamp: 'TudorGirba 1/20/2016 23:27'!
extensionsString
	| methods title |
	^ String streamContents: [ :s |
		((GTInspector inspectorExtendingMethods groupedBy: #methodClass)
		associations sorted: [ :a :b | a key name < b key name ]) 
		do: [ :classMethodsAssociation |
			s nextPutAll: classMethodsAssociation key name; cr.
			methods := classMethodsAssociation value.
			methods do: [ :each | 
				title := each ast allChildren 
					detect: [ :node | node isMessage and: [ node selector = #title: ] ]
					ifFound: [ :node | node arguments first value. ]
					ifNone: [ each selector ].
				title isString ifFalse: [ title := each selector ].
				s 
					nextPutAll: '- '; 
					nextPutAll: title; 
					nextPutAll: ' ('; 
					nextPutAll: each selector; 
					nextPut: $);
					cr ].
			s cr ] ]! !

!GTInspectorHelp class methodsFor: 'accessing' stamp: 'TudorGirba 1/21/2016 16:45'!
key
	^ self name! !

!GTInspectorHelp class methodsFor: 'documentation' stamp: 'TudorGirba 1/20/2016 16:47'!
overview
	^ HelpTopic 
		title: 'Overview'
		contents: 
'GT Inspector is a moldable inspector that is part of Pharo and was developed as part of the Glamorous Toolkit project (http://gt.moosetechnology.org)'! !

!GTInspectorHelp class methodsFor: 'accessing' stamp: 'AlexandreBergel 5/2/2016 14:55'!
pages
	^#(overview table roassal extensions)! !

!GTInspectorHelp class methodsFor: 'documentation' stamp: 'AlexandreBergel 12/16/2015 13:42'!
roassal
	^ HelpTopic 
		title: 'Roassal'
		contents: 
'Roassal is a visualization engine. GTInspector may uses Roassal to craft a visual representation for a given object.

First of all, you need to load Roassal. You can verify is Roassal is loaded why opening the World menu: a Roassal entry should be found. If no, then Roassal is not loaded. You can load Roassal2 from the Catalog browser.

You can then create a visual inspection by defining the following method in a class:
-=-=-=-=
gtInspectorViewIn: composite
	<gtInspectorPresentationOrder: -10>
	composite roassal2
		title: ''View'';
		initializeView: [ | v |
			v := RTView new.
			...
			v ]
-=-=-=-=

As an example, consider the following method you can defined on the class OrderedCollection:
-=-=-=-=
gtInspectorIdentityIn: composite
	<gtInspectorPresentationOrder: -10>
	composite roassal2
		title: ''Identity'';
		initializeView: [
			| b |
			b := RTMondrian new.
			b nodes: self.
			b normalizer distinctColor.
			b layout grid; gapSize: 1.
			b ]
-=-=-=-=

Inspecting the expression:
((1 to: 5), (1 to: 5), (1 to: 5)) asOrderedCollection

will give a color to each number.

You can give the method name you wish. The pragma: <gtInspectorPresentationOrder: -10> is essential as it is a marker for the GTInspector framework. -10 indicates the location of the tab. The tab ''Raw'' has a priority of 10. So giving a priority lower than 10 will position your visualization on the left of ''Raw''. Greater than 10 on the right of it.
'! !

!GTInspectorHelp class methodsFor: 'documentation' stamp: 'AlexandreBergel 12/16/2015 13:56'!
table
	^ HelpTopic 
		title: 'Table'
		contents: 
'A table is a presentation that simply list elements. For example, you can create a table presentation for the class OrderedCollection by defining the following method in OrderedCollection:
-=-=-=-=
gtInspectorTableIn: composite
	<gtInspectorPresentationOrder: -15>
	composite table
		title: ''Table'';
		display: [ self collect: #asString ]
-=-=-=-=

Inspect the expression:
((1 to: 5), (1 to: 5), (1 to: 5)) asOrderedCollection

You will have a list of numbers.

'! !

Object subclass: #GTInspectorPresentationFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorPresentationFilter commentStamp: 'AndreiChis 1/27/2015 23:31' prior: 0!
A GTInspectorPresentationFilter is the root class for filters that can limit the number if visible presentations for an object.

Subclasses need to implement  #shouldDisplayPresentationCreateBy: to indicate whether or not a give presentation should be displayed.
!

GTInspectorPresentationFilter subclass: #GTInspectorMethodListFilter
	instanceVariableNames: 'allMethodSignatures selectedMethodSignatures'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorMethodListFilter commentStamp: 'AndreiChis 1/27/2015 23:30' prior: 0!
GTInspectorMethodListFilter implements a filtering strategy for presentations based on method signatures (method selector and class).

The filter returns true for a method if the signature of that method is selected in the filter. To be taken into account during filtering a signature must be first added to the filter and the selected. Only signatures that have been first added can be selected. !

!GTInspectorMethodListFilter class methodsFor: 'examples' stamp: 'AndreiChis 1/27/2015 23:30'!
exampleASTBrowser
	"An inspector showing a browser for navigating through AST nodes."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorMethodListFilter new
		addAndSelectSignatureFor: CompiledMethod>>#gtInspectorASTIn:;
		addAndSelectSignatureFor: RBProgramNode>>#gtInspectorSourceCodeIn:).
	inspector openOn: Morph>>#initialize.! !

!GTInspectorMethodListFilter class methodsFor: 'examples' stamp: 'AndreiChis 1/27/2015 23:30'!
exampleCodeBrowser
	"An inspector showing only the Meta presentation for all objects"
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorMethodListFilter new
		addAndSelectSignatureFor: Object>>#gtInspectorMetaIn:).
	inspector openOn: Morph new.! !

!GTInspectorMethodListFilter class methodsFor: 'examples' stamp: 'AndreiChis 1/27/2015 23:30'!
exampleFileBrowser
	"An inspector for browsing directories and files."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorMethodListFilter new
		addAndSelectSignatureFor: FileReference>>#gtInspectorStScriptIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorPngIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorJpegIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorItemsIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorGifIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorContentsIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorFuelIn:).
	inspector openOn: FileSystem workingDirectory.! !

!GTInspectorMethodListFilter class methodsFor: 'examples' stamp: 'AndreiChis 1/27/2015 23:30'!
exampleFullInspector
	"An inspector showing all presentations for all objects"
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorMethodListFilter new
		addAndSelectAllSignatures).
	inspector openOn: Morph new.! !

!GTInspectorMethodListFilter class methodsFor: 'examples' stamp: 'AndreiChis 3/6/2015 10:33'!
exampleHelpBrowser
	"An inspector for browsing Help pages.
	
	self exampleHelpBrowser
	"
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorMethodListFilter new
		addAndSelectSignatureFor: HelpTopic>>#gtInspectorContentsIn:;
		addAndSelectSignatureFor: HelpTopic>>#gtInspectorSubtopicsIn:).
	inspector openOn: SystemHelp asHelpTopic.! !

!GTInspectorMethodListFilter class methodsFor: 'examples' stamp: 'AndreiChis 1/27/2015 23:30'!
exampleRawInspector
	"An inspector showing only the Raw presentation for all objects"
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorMethodListFilter new
		addAndSelectSignatureFor: Object>>#gtInspectorRawIn:).
	inspector openOn: Morph new.! !

!GTInspectorMethodListFilter methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:28'!
addAllSignatures
	
	self allPresentationMethods do: [ :aMethod | self addSignatureFor: aMethod ] ! !

!GTInspectorMethodListFilter methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:33'!
addAndSelectAllSignatures
	
	self allPresentationMethods do: [ :aMethod | self addAndSelectSignatureFor: aMethod ] ! !

!GTInspectorMethodListFilter methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:46'!
addAndSelectSignatureFor: aMethod
	|signature|
	
	signature := self signatureFor: aMethod.
	(allMethodSignatures includes: signature) ifFalse: [ allMethodSignatures add: signature ].
	(selectedMethodSignatures includes: signature) ifFalse: [ selectedMethodSignatures add: signature ].
	! !

!GTInspectorMethodListFilter methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:24'!
addSignatureFor: aMethod
	|signature|
	
	signature := self signatureFor: aMethod.
	(allMethodSignatures includes: signature) ifFalse: [ allMethodSignatures add: signature ].! !

!GTInspectorMethodListFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:32'!
allPresentationMethods

	^ (PragmaCollector allSystemPragmas select: [ :pragma | 
		pragma key == self inspectorPragmaKey ]) collect: [ :aPragma | aPragma method ]! !

!GTInspectorMethodListFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 12:06'!
changeFilterRequestFor: aGTInspector

	self
		openListChooseLabeled: 'All available presentations' 
		forInspector: aGTInspector
		with: allMethodSignatures 
		select: selectedMethodSignatures 
		format: [ :aSignature | self printSignature: aSignature ]
		actionSelector: #replaceSelectedSignaturesWith:in:! !

!GTInspectorMethodListFilter methodsFor: 'initialization' stamp: 'AndreiChis 12/13/2014 11:43'!
initialize
	super initialize.
	allMethodSignatures := OrderedCollection new.
	selectedMethodSignatures := OrderedCollection new.! !

!GTInspectorMethodListFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:31'!
inspectorPragmaKey

	^ #gtInspectorPresentationOrder:! !

!GTInspectorMethodListFilter methodsFor: 'printing' stamp: 'AndreiChis 12/13/2014 12:06'!
printSignature: aSignature

	^ aSignature first name, '>>', aSignature second asString! !

!GTInspectorMethodListFilter methodsFor: 'callbacks' stamp: 'AndreiChis 12/13/2014 11:46'!
replaceSelectedSignaturesWith: aList in: aGTInspector

	selectedMethodSignatures := aList.
	aGTInspector update! !

!GTInspectorMethodListFilter methodsFor: 'testing' stamp: 'AliakseiSyrel 2/9/2017 00:19'!
shouldDisplayPresentationCreatedBy: aMethod

	^ selectedMethodSignatures anySatisfy: [ :aSignature | 
		aSignature first = aMethod methodClass and: [ aSignature second = aMethod selector ] ]! !

!GTInspectorMethodListFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 12:03'!
signatureFor: aMethod

	^ {aMethod methodClass . aMethod selector}! !

GTInspectorPresentationFilter subclass: #GTInspectorNoneFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorNoneFilter commentStamp: 'AliakseiSyrel 2/9/2017 00:16' prior: 0!
I do not filter presentation and allow all of them to be displayed!

!GTInspectorNoneFilter methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/9/2017 00:14'!
changeFilterRequestFor: aGTInspector
	"do nothing"! !

!GTInspectorNoneFilter methodsFor: 'testing' stamp: 'AliakseiSyrel 2/9/2017 00:19'!
shouldDisplayPresentationCreatedBy: aMethod
	"display all of them"
	^ true! !

!GTInspectorPresentationFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/9/2014 22:38'!
changeFilterRequestFor: aGTInspector

	self subclassResponsibility! !

!GTInspectorPresentationFilter methodsFor: 'private' stamp: 'AndreiChis 12/13/2014 12:04'!
openListChooseLabeled: sString forInspector: aGTInspector with: allElements select: selectedElements format: aBlockOrSelector actionSelector: aSymbol
	|treeModel|
	
	treeModel := MorphTreeModel new.
	treeModel
		 beCheckList;
		 rootItems: allElements;
		 wrapBlockOrSelector: aBlockOrSelector;
		 autoMultiSelection: true.
	 (treeModel openDialogWindowIn: World title: sString selectedtems: selectedElements) ifNotNil: [ :selection | 
		self perform: aSymbol with: selection with: aGTInspector ]! !

!GTInspectorPresentationFilter methodsFor: 'testing' stamp: 'AliakseiSyrel 2/9/2017 00:19'!
shouldDisplayPresentationCreatedBy: aMethod

	self subclassResponsibility! !

GTInspectorPresentationFilter subclass: #GTInspectorTagFilter
	instanceVariableNames: 'allTags selectedTags defaultTag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorTagFilter commentStamp: 'AndreiChis 1/27/2015 23:31' prior: 0!
GTInspectorTagFilter implements a filtering strategy for presentations based on tags.

A tag is defined as the parameter of the prama <gtInspectorTag:>. A method can have one or more tags.
The filter returns true for a method if that method contains at least a tag selected in the filter. To be taken into account during filtering a tag must be first added to the filter and the selected. Only tags that have been first added can be selected.  

If a method does not define any tags the filter threats that method as having a single tag stored in the #defaultTag instance variable. The default defaultTag is #custom. The #basic tag  groups the Raw and Meta presentations. Inspect 'GTInspectorTagFilter new allPragmasWithTag' to view all pragmas creating the tags.!

!GTInspectorTagFilter class methodsFor: 'examples' stamp: 'AndreiChis 1/27/2015 23:30'!
exampleBasicInspector
	"An inspector showing only the basic presentations Raw and Meta for all object."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorTagFilter new
		addAndSelectTag: #basic).
	inspector openOn: Morph new.! !

!GTInspectorTagFilter class methodsFor: 'examples' stamp: 'AndreiChis 1/27/2015 23:30'!
exampleCustomInspector
	"An inspector showing only the custom presentations for all object (excluding Raw and Meta)."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorTagFilter new
		defaultTag: #custom;
		addAndSelectTag: #custom).
	inspector openOn: Morph new.! !

!GTInspectorTagFilter class methodsFor: 'examples' stamp: 'AndreiChis 1/27/2015 23:30'!
exampleExampleBrowser
	"A browser for viewing examples"
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorTagFilter new
		addAndSelectTag: #examples).
	inspector openOn: Dictionary.! !

!GTInspectorTagFilter class methodsFor: 'examples' stamp: 'AndreiChis 1/27/2015 23:30'!
exampleFullInspector
	"An inspector showing all the available presentations for all objects."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationFilter: (GTInspectorTagFilter new
		defaultTag: #custom;
		addAndSelectTag: #custom;
		addAndSelectAllTags).
	inspector openOn: Morph new.! !

!GTInspectorTagFilter methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:15'!
addAllTags
	
	self allTags do: [ :aTag | self addTag: allTags ] ! !

!GTInspectorTagFilter methodsFor: 'adding' stamp: 'AndreiChis 1/14/2015 15:21'!
addAndSelectAllTags
	
	self allTags do: [ :aTag | self addAndSelectTag: aTag ] ! !

!GTInspectorTagFilter methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:16'!
addAndSelectTag: aSymbol

	self addTag: aSymbol.
	(selectedTags includes: aSymbol) ifFalse:  [selectedTags add: aSymbol]! !

!GTInspectorTagFilter methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:16'!
addTag: aSymbol

	(allTags includes: aSymbol) ifFalse:  [allTags add: aSymbol].! !

!GTInspectorTagFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:13'!
allPragmasWithTag

	^ PragmaCollector allSystemPragmas select: [ :pragma | pragma key == self tagPragmaKey ]! !

!GTInspectorTagFilter methodsFor: 'accessing' stamp: 'AndreiChis 1/14/2015 15:18'!
allTags

	^ (self allPragmasWithTag collect: [ :pragma | 
		pragma argumentAt: 1 ]) asSet sorted: [ :a :b | a < b ]! !

!GTInspectorTagFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 12:05'!
changeFilterRequestFor: aGTInspector

	self
		openListChooseLabeled: 'All available tags' 
		forInspector: aGTInspector
		with: allTags 
		select: selectedTags 
		format: #printString
		actionSelector: #replaceSelectedTagsWith:in:
! !

!GTInspectorTagFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 00:38'!
defaultTag: aSymbol

	defaultTag := aSymbol! !

!GTInspectorTagFilter methodsFor: 'initialization' stamp: 'AndreiChis 12/13/2014 10:59'!
initialize
	super initialize.
	
	allTags := OrderedCollection new.
	selectedTags := OrderedCollection new.
	defaultTag := #custom! !

!GTInspectorTagFilter methodsFor: 'callbacks' stamp: 'AndreiChis 12/13/2014 11:40'!
replaceSelectedTagsWith: aListOfSymbols in: aGTInspector

	selectedTags := aListOfSymbols.
	aGTInspector update.! !

!GTInspectorTagFilter methodsFor: 'testing' stamp: 'AliakseiSyrel 2/9/2017 00:19'!
shouldDisplayPresentationCreatedBy: aMethod

	^ (self tagsFrom: aMethod) includesAny: selectedTags! !

!GTInspectorTagFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:11'!
tagPragmaKey

	^ #gtInspectorTag:! !

!GTInspectorTagFilter methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:11'!
tagsFrom: aMethod

	^ (aMethod pragmas select: [ :each | each key == self tagPragmaKey ])
		ifEmpty: [ defaultTag ifNil: [ {} ] ifNotNil: [ {defaultTag} ] ]
		ifNotEmpty: [ :currentPragmas | currentPragmas collect: [ :each | each argumentAt: 1 ] ]! !

Object subclass: #GTInspectorVariableNode
	instanceVariableNames: 'hostObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorVariableNode commentStamp: 'AndreiChis 2/27/2017 11:19' prior: 0!
I model an attribute of an object in the GTInspector. I am used in the Raw presentation of an Object. My main responsibility is to associate a variable name of a host object with its value. 

Public API and Key Messages

- hostObject return the object (host) holding the  attribute that I represent;
- label return a string label of the attribute, which may be an instance variable name or a dynamically generated name for dynamic attributes;
- value return the value  of the attribute; catches any errors and returns nil;
- rawValue return the value of the attribute without any catch for errors;
- key return an object used as unique key to identify an attribute, it may be an integer for indexed variables or a string for dynamic ones.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	hostObject:		<Object>!

GTInspectorVariableNode subclass: #GTInspectorContextIndexedTempNode
	instanceVariableNames: 'tempIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorContextIndexedTempNode commentStamp: 'AndreiChis 3/4/2017 14:01' prior: 0!
I am a variable node for representing an  indexed variable stored in a  Context object that does not have a name.  These are values that are pushed and popped on the stack when executing message sends. For example, the parameters neede to perform a message send are pushed in a Context object.!

!GTInspectorContextIndexedTempNode class methodsFor: 'as yet unclassified' stamp: 'AndreiChis 3/4/2017 12:07'!
hostObject: aContext tempIndex: anInteger
	^ (self hostObject: aContext) 
		tempIndex: anInteger;
		yourself! !

!GTInspectorContextIndexedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 12:08'!
label
	^ self tempIndex! !

!GTInspectorContextIndexedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 12:07'!
rawValue
	^ self hostObject at: self tempIndex! !

!GTInspectorContextIndexedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 12:04'!
tempIndex
	^ tempIndex! !

!GTInspectorContextIndexedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 12:04'!
tempIndex: anObject
	tempIndex := anObject! !

GTInspectorVariableNode subclass: #GTInspectorContextNamedTempNode
	instanceVariableNames: 'tempName tempIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorContextNamedTempNode commentStamp: 'AndreiChis 3/4/2017 14:01' prior: 0!
I am a variable node for representing a  temporary  variable stored in a  Context object that has a name attached. This includes  local variables and method parameters.!

!GTInspectorContextNamedTempNode class methodsFor: 'instance creation' stamp: 'AndreiChis 3/3/2017 19:34'!
hostObject: aContext tempName: aString tempIndex: anInteger
	^ (self hostObject: aContext) 
		tempName: aString;
		tempIndex: anInteger;
		yourself! !

!GTInspectorContextNamedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 11:33'!
errorMessage
	^ 'error obtaining field value'! !

!GTInspectorContextNamedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 13:51'!
key
	^ self tempName! !

!GTInspectorContextNamedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/3/2017 19:26'!
label
	^ '[' , self key , ']'! !

!GTInspectorContextNamedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/3/2017 19:26'!
rawValue
	^ self hostObject debuggerMap namedTempAt: tempIndex in: self hostObject! !

!GTInspectorContextNamedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/3/2017 19:25'!
tempIndex
	^ tempIndex! !

!GTInspectorContextNamedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/3/2017 19:25'!
tempIndex: anObject
	tempIndex := anObject! !

!GTInspectorContextNamedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/3/2017 19:24'!
tempName
	^ tempName! !

!GTInspectorContextNamedTempNode methodsFor: 'accessing' stamp: 'AndreiChis 3/3/2017 19:24'!
tempName: aString
	tempName := aString! !

GTInspectorVariableNode subclass: #GTInspectorDynamicNode
	instanceVariableNames: 'label value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorDynamicNode commentStamp: 'AndreiChis 2/20/2017 11:45' prior: 0!
I am a variable node for representing any dynamically computed attribute that should be displayed in the Raw presentation. I should not be used for slots or indexed variables.!

!GTInspectorDynamicNode class methodsFor: 'instance creation' stamp: 'AndreiChis 2/26/2017 18:43'!
hostObject: anObject label: aString value: anotherObject
	^ (self hostObject: anObject) 
		label: aString;
		rawValue: anotherObject;
		yourself! !

!GTInspectorDynamicNode methodsFor: 'accessing' stamp: 'AndreiChis 2/7/2017 14:53'!
key
	^ label! !

!GTInspectorDynamicNode methodsFor: 'accessing' stamp: 'AndreiChis 2/7/2017 13:44'!
label
	^ label! !

!GTInspectorDynamicNode methodsFor: 'accessing' stamp: 'AndreiChis 2/7/2017 13:44'!
label: aString
	label := aString! !

!GTInspectorDynamicNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:43'!
rawValue
	^ value! !

!GTInspectorDynamicNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:43'!
rawValue: anObject
	value := anObject! !

GTInspectorVariableNode subclass: #GTInspectorIndexedNode
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorIndexedNode commentStamp: 'AndreiChis 2/27/2017 11:19' prior: 0!
I am a variable node for representing the indexed variables of an object. I used the index of a variable as a key. I extract the value from the host object every time #rawValue is called.!

!GTInspectorIndexedNode class methodsFor: 'instance creation' stamp: 'AliakseiSyrel 2/8/2017 23:26'!
hostObject: anObject index: index
	^ (self hostObject: anObject)
		index: index;
		yourself! !

!GTInspectorIndexedNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 11:33'!
errorMessage
	^ 'error obtaining indexed attribute value'! !

!GTInspectorIndexedNode methodsFor: 'accessing' stamp: 'AndreiChis 2/7/2017 13:40'!
index
	^ index! !

!GTInspectorIndexedNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 15:53'!
index: anInteger
	index := anInteger! !

!GTInspectorIndexedNode methodsFor: 'accessing' stamp: 'AndreiChis 2/7/2017 14:52'!
key
	^ self index! !

!GTInspectorIndexedNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:41'!
rawValue
	^ self hostObject basicAt: index! !

GTInspectorVariableNode subclass: #GTInspectorProtoObjectNode
	instanceVariableNames: 'index slotName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorProtoObjectNode commentStamp: 'AndreiChis 2/27/2017 11:20' prior: 0!
I am a variable node for representing the slots of objects that subclass ProtoObject directly and not Object. My main responsability is to extract the value of a slot using thisContext and not a direct message send to the object.  I used the name of a slot as a key. I extract the value from the host object every time #rawValue is called.!

!GTInspectorProtoObjectNode class methodsFor: 'instance creation' stamp: 'AliakseiSyrel 2/8/2017 23:26'!
hostObject: anObject index: index slotName: aSlotName
	^ (self hostObject: anObject)
		index: index slotName: aSlotName;
		yourself! !

!GTInspectorProtoObjectNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 15:53'!
index: anIndex slotName: aSlotName
	index := anIndex.
	slotName := aSlotName.! !

!GTInspectorProtoObjectNode methodsFor: 'accessing' stamp: 'AndreiChis 2/7/2017 14:52'!
key
	^ slotName! !

!GTInspectorProtoObjectNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:41'!
rawValue 
	^ thisContext object: self hostObject instVarAt: index ! !

GTInspectorVariableNode subclass: #GTInspectorSelfNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorSelfNode commentStamp: 'AndreiChis 2/20/2017 11:39' prior: 0!
I model the  `self` variable. This the value of this variable is always the host object. !

!GTInspectorSelfNode methodsFor: 'composition' stamp: 'AliakseiSyrel 2/8/2017 23:43'!
childrenNodesFor: aVariablesBrowser
	"Self node does not have any subvariables, just return empty collection"
	<return: #Collection of: #GTInspectorVariableNode>

	^ #()! !

!GTInspectorSelfNode methodsFor: 'accessing' stamp: 'AndreiChis 2/7/2017 14:52'!
key
	^ 'self'! !

!GTInspectorSelfNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:42'!
rawValue
	^ self hostObject! !

GTInspectorVariableNode subclass: #GTInspectorSlotNode
	instanceVariableNames: 'slot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorSlotNode commentStamp: 'AndreiChis 2/27/2017 11:20' prior: 0!
I am a variable node for representing the slots of an object. I used the name of a slot as a key. I extract the value from the host object every time #rawValue is called.!

!GTInspectorSlotNode class methodsFor: 'instance creation' stamp: 'AliakseiSyrel 2/8/2017 23:26'!
hostObject: anObject slot: aSlot
	^ (self hostObject: anObject)
		slot: aSlot;
		yourself! !

!GTInspectorSlotNode methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/8/2017 23:46'!
browsingActionsForNode

	^ {
			GLMGenericAction new 
				action: [ self systemNavigation browseAllAccessesTo: self label from: self hostObject class ]; 
				title: 'List methods using this variable'.
			GLMGenericAction new 
				action: [ self systemNavigation browseAllStoresInto: self label from: self hostObject class ]; 
				title: 'List methods storing into this variable'.		
		}! !

!GTInspectorSlotNode methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/8/2017 23:46'!
editingActionForNode

	^ GLMGenericAction new 
		action: [ self inform: 'To be implemented' ]; 
		title: 'Edit value'! !

!GTInspectorSlotNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 11:33'!
errorMessage
	^ 'error obtaining attribute value'! !

!GTInspectorSlotNode methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/8/2017 23:45'!
inspectorActionsForNode
	"Return a collection of available inspector actions associated with a variable node"
	<return: #Collection of: #GLMAction>

	^ 
		{ self editingActionForNode },
		self browsingActionsForNode! !

!GTInspectorSlotNode methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/8/2017 23:16'!
key
	^ self slot name! !

!GTInspectorSlotNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:42'!
rawValue
	^ self hostObject readSlot: slot! !

!GTInspectorSlotNode methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/8/2017 23:13'!
slot
	^ slot! !

!GTInspectorSlotNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 15:54'!
slot: aSlot

	self
		assert: [ aSlot isNotNil ]
		description: [ 'Slot can not be nil' ].

	slot := aSlot.! !

!GTInspectorVariableNode class methodsFor: 'instance creation' stamp: 'AndreiChis 2/7/2017 14:24'!
hostObject: anObject 
	^ self new
		hostObject: anObject;
		yourself! !

!GTInspectorVariableNode methodsFor: 'composition' stamp: 'AndreiChis 2/26/2017 18:38'!
childrenNodesFor: aVariablesBrowser
	"Return a collection of subvariable nodes to be displayed in a given raw variables browser.
	Note: by default we dispatch to browser and let it perform collection"
	<return: #Collection of: #GTInspectorVariableNode>

	^ aVariablesBrowser childrenVariableNodesForNode: self! !

!GTInspectorVariableNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 11:33'!
errorMessage
	^ 'error obtaining value'! !

!GTInspectorVariableNode methodsFor: 'accessing' stamp: 'AndreiChis 2/7/2017 13:42'!
hostObject
	^ hostObject! !

!GTInspectorVariableNode methodsFor: 'accessing' stamp: 'AndreiChis 2/12/2017 15:27'!
hostObject: anObject

	"
	Allow nil objects for the moment. Normally the inspector should be able to inspect nil.
	self
		assert: [ anObject isNotNil ]
		description: [ 'Host object must not be nil' ]."

	self
		assert: [ hostObject isNil ]
		description: [ 'Host can be assigned only once' ].

	hostObject := anObject! !

!GTInspectorVariableNode methodsFor: 'actions' stamp: 'AliakseiSyrel 2/8/2017 23:44'!
inspectorActionsForNode
	"Return a collection of available inspector actions associated with a variable node"
	<return: #Collection of: #GLMAction>

	^ #()! !

!GTInspectorVariableNode methodsFor: 'accessing' stamp: 'AliakseiSyrel 2/8/2017 23:24'!
key
	"Answer the key of this object variable (slot, indexed attribute, computed value)."
	^ self subclassResponsibility! !

!GTInspectorVariableNode methodsFor: 'accessing' stamp: 'AndreiChis 2/7/2017 14:51'!
label
	"Answer the label of this object variable (slot, indexed attribute, computed value)."
	^ self key asString! !

!GTInspectorVariableNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:40'!
rawValue
	"Answer the object value of this object variable (slot, indexed attribute, computed value)."
	^ self subclassResponsibility! !

!GTInspectorVariableNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:41'!
value
	"Answer the object value of this object variable (slot, indexed attribute, computed value).
	Return nil in case there is an error obtaining the value"
	^ [ self rawValue ] on: Error do: [ nil ]! !

GTInspectorVariableNode subclass: #GTInspectorWrapperNode
	instanceVariableNames: 'label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorWrapperNode commentStamp: 'AndreiChis 2/27/2017 11:51' prior: 0!
I am a node that wraps in the host object another node and delegates all API methods to the wrapped node. I can further provide a different label for the wrapped node.!

!GTInspectorWrapperNode class methodsFor: 'instance creation' stamp: 'AndreiChis 2/26/2017 18:45'!
hostObject: anObject label: aString
	^ (self hostObject: anObject) 
		label: aString;
		yourself! !

!GTInspectorWrapperNode methodsFor: 'composition' stamp: 'AndreiChis 2/26/2017 18:37'!
childrenNodesFor: aVariablesBrowser
	^ self hostObject childrenNodesFor: aVariablesBrowser! !

!GTInspectorWrapperNode methodsFor: 'accessing' stamp: 'AndreiChis 3/4/2017 12:36'!
errorMessage
	^ self hostObject errorMessage! !

!GTInspectorWrapperNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:37'!
inspectorActionsForNode
	^ self hostObject inspectorActionsForNode! !

!GTInspectorWrapperNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:46'!
key
	^ label ifNil: [ self hostObject key ]! !

!GTInspectorWrapperNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:46'!
label: aString
	label := aString! !

!GTInspectorWrapperNode methodsFor: 'accessing' stamp: 'AndreiChis 2/26/2017 18:42'!
rawValue
	^ self hostObject rawValue! !

Object subclass: #GTObjectPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTObjectPrinter commentStamp: 'TudorGirba 1/18/2014 14:29' prior: 0!
This is a utility class for printing objects to Text or String.!

!GTObjectPrinter class methodsFor: 'converting' stamp: 'AndreiChis 2/12/2017 15:17'!
asNonTruncatedTextFrom: anObject
	"I return a representation of the receiver in which all lines breaks 
	are replaced by spaces. I return a String unless there is an error 
	printing the object. In this case I return a Text highlighted in red."
	^ [ anObject asString replaceAll: String cr with: String space ] 
			on: Error 
			do: [ Text string: 'error printing' attribute: TextColor red ]! !

!GTObjectPrinter class methodsFor: 'converting' stamp: 'AndreiChis 2/12/2017 15:17'!
asTruncatedTextFrom: anObject
	"I return a truncated representation of the receiver in which all lines breaks 
	are replaced by spaces. I return a String unless there is an error printing the
	object. In this case I return a Text highlighted in red."
	^ [ (anObject gtDisplayString copyReplaceAll: String cr with: String space) replaceAll: String lf with: String space ]
		on: Error
		do: [ Text string: 'error printing' attribute: TextColor red ]! !

!GTObjectPrinter methodsFor: 'printing' stamp: 'AndreiChis 2/20/2017 11:06'!
asNonTruncatedTextFrom: anObject
	^ self class asNonTruncatedTextFrom: anObject! !

!GTObjectPrinter methodsFor: 'printing' stamp: 'AndreiChis 2/20/2017 11:06'!
asTruncatedTextFrom: anObject
	^ self class asTruncatedTextFrom: anObject! !

Object subclass: #GTSnippets
	instanceVariableNames: 'instance snippets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

GTSnippets class
	instanceVariableNames: 'instance'!

!GTSnippets commentStamp: '<historical>' prior: 0!
GTSnippets instance!

GTSnippets class
	instanceVariableNames: 'instance'!

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 14:55'!
instance
	^ instance ifNil: [ instance := self new ]! !

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 2/2/2014 14:05'!
reset
	instance := nil! !

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 15:00'!
snippetAt: anObject
	^ self instance snippetAt: anObject! !

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 15:10'!
snippetAt: anObject put: aString
	^ self instance snippetAt: anObject put: aString! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 12/30/2013 23:44'!
snippetAt: anObject
	^ self snippets 
		at: anObject 
		ifAbsent: [ 'self' ]! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 12/30/2013 23:44'!
snippetAt: anObject put: aString
	^ self snippets 
		at: anObject 
		put: aString! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 5/26/2014 12:39'!
snippetAtOrEmpty: anObject
	^ self snippets 
		at: anObject 
		ifAbsent: [ '' ]! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 14:59'!
snippets 
	^ snippets ifNil: [ snippets := IdentityDictionary new ]! !

!Object methodsFor: '*GT-Inspector' stamp: 'TudorGirba 12/23/2013 09:12'!
gtDisplayOn: stream
	"This offers a means to customize how the object is shown in the inspector"
	^ self printOn: stream! !

!Object methodsFor: '*GT-Inspector' stamp: 'AndreiChis 2/26/2017 19:32'!
gtDisplayString
	"This offers a means to customize how the object is shown in the inspector.
	Do not override this method. Override gtDisplayOn: instead"
		
	| limitedString limit |
	limit := 1000.
	limitedString := String streamContents: [:s | self gtDisplayOn: s] limitedTo: limit.
	limitedString size < limit ifTrue: [^ limitedString].
	^ limitedString , ' ...'! !

!Object methodsFor: '*GT-Inspector' stamp: 'AndreiChis 4/20/2016 17:54'!
gtInspectorHash

	^ self identityHash.! !

!Object methodsFor: '*GT-Inspector' stamp: 'AndreiChis 2/26/2017 18:03'!
gtInspectorIcon
	^ self iconOrThumbnailOfSize: 16! !

!Object methodsFor: '*GT-Inspector' stamp: 'TudorGirba 6/28/2014 22:18'!
gtInspectorInterestingObject
	^ self! !

!Object methodsFor: '*GT-Inspector' stamp: 'TudorGirba 1/4/2016 20:59'!
gtInspectorRawIn: composite
	"This is the most basic presentation showing the state of the object"
	^ composite custom: GTObjectVariablesBrowser new! !

!Object methodsFor: '*GT-Inspector' stamp: 'AndreiChis 2/26/2017 19:37'!
gtInspectorVariableNodesIn: aCollection
	"This is a helper method that adds to the given collection all nodes modeling named and 
	indexed attributes for the current object.
	Subclasses can override it to specialize what appears in the variables presentation."
	| indexableDisplayLimit top bottom topLimit bottomLimit |
	
	indexableDisplayLimit := GTInspector indexableDisplayLimit.        
	top := 1.
	bottom := self basicSize.
	topLimit    := indexableDisplayLimit min: bottom.
	bottomLimit := indexableDisplayLimit max: (bottom - indexableDisplayLimit).

	top to: topLimit do: [ :index | 
		aCollection add: (GTInspectorIndexedNode hostObject: self index: index) ].
	
	bottomLimit + 1 to: bottom do: [ :index |
		aCollection add: (GTInspectorIndexedNode hostObject: self index: index) ].
	
	aCollection
		addAll: ((self class allSlots collect: [ :slot | 
			GTInspectorSlotNode hostObject: self slot: slot ]))! !

!Object methodsFor: '*GT-Inspector' stamp: 'AndreiChis 2/7/2017 14:54'!
gtInspectorVariableValuePairs
	"This is a helper method that returns a collection of 
		variable_name -> value
	for the current object.
	Subclasses can override it to specialize what appears in the variables presentation"
	| bindings |
	bindings := OrderedCollection new.
	self gtInspectorVariableNodesIn: bindings.
	^ bindings collect: [ :aNode | aNode key -> aNode value ]! !

GTInspector initialize!
